@@ -38,17 +38,17 @@ static const float PIo2[] = {
 #else
 static float PIo2[] = {
 #endif
-  1.5703125000e+00, /* 0x3fc90000 */
-  4.5776367188e-04, /* 0x39f00000 */
-  2.5987625122e-05, /* 0x37da0000 */
-  7.5437128544e-08, /* 0x33a20000 */
-  6.0026650317e-11, /* 0x2e840000 */
-  7.3896444519e-13, /* 0x2b500000 */
-  5.3845816694e-15, /* 0x27c20000 */
-  5.6378512969e-18, /* 0x22d00000 */
-  8.3009228831e-20, /* 0x1fc40000 */
-  3.2756352257e-22, /* 0x1bc60000 */
-  6.3331015649e-25, /* 0x17440000 */
+  1.5703125000e+00f, /* 0x3fc90000 */
+  4.5776367188e-04f, /* 0x39f00000 */
+  2.5987625122e-05f, /* 0x37da0000 */
+  7.5437128544e-08f, /* 0x33a20000 */
+  6.0026650317e-11f, /* 0x2e840000 */
+  7.3896444519e-13f, /* 0x2b500000 */
+  5.3845816694e-15f, /* 0x27c20000 */
+  5.6378512969e-18f, /* 0x22d00000 */
+  8.3009228831e-20f, /* 0x1fc40000 */
+  3.2756352257e-22f, /* 0x1bc60000 */
+  6.3331015649e-25f, /* 0x17440000 */
 };
 
 #ifdef __STDC__
@@ -56,10 +56,10 @@ static const float
 #else
 static float			
 #endif
-zero   = 0.0,
-one    = 1.0,
-two8   =  2.5600000000e+02, /* 0x43800000 */
-twon8  =  3.9062500000e-03; /* 0x3b800000 */
+zero   = 0.0f,
+one    = 1.0f,
+two8   =  2.5600000000e+02f, /* 0x43800000 */
+twon8  =  3.9062500000e-03f; /* 0x3b800000 */
 
 #ifdef __STDC__
 	int __kernel_rem_pio2f(float *x, float *y, int e0, int nx, int prec, const __int32_t *ipio2) 
