@@ -79,8 +79,8 @@
 // SDRAM
 #define MICROPY_HW_SDRAM_SIZE  (64 / 8 * 1024 * 1024)  // 64 Mbit
 #define MICROPY_HW_SDRAM_STARTUP_TEST             (1)
-#define MICROPY_HEAP_START              sdram_start()
-#define MICROPY_HEAP_END                sdram_end()
+#define MICROPY_HEAP_START  ((sdram_valid) ? sdram_start() : &_heap_start)
+#define MICROPY_HEAP_END    ((sdram_valid) ? sdram_end() : &_heap_end)
 
 // Timing configuration for 90 Mhz (11.90ns) of SD clock frequency (180Mhz/2)
 #define MICROPY_HW_SDRAM_TIMING_TMRD        (2)
