@@ -2,7 +2,7 @@
 #include "lib/utils/interrupt_char.h"
 
 static inline mp_uint_t mp_hal_ticks_us(void) {
-    return SYS_CLOCK_HW_CYCLES_TO_NS(k_cycle_get_32()) / 1000;
+    return k_cyc_to_ns_floor64(k_cycle_get_32()) / 1000;
 }
 
 static inline mp_uint_t mp_hal_ticks_ms(void) {
@@ -21,7 +21,12 @@ static inline void mp_hal_delay_us(mp_uint_t delay) {
 }
 
 static inline void mp_hal_delay_ms(mp_uint_t delay) {
-    k_sleep(delay);
+    k_msleep(delay);
+}
+
+static inline uint64_t mp_hal_time_ns(void) {
+    // Not currently implemented.
+    return 0;
 }
 
 #define mp_hal_delay_us_fast(us)   (mp_hal_delay_us(us))
